<?php
class VLCServer {
	/**
	* @access private
	* @var int
	*/
	private $lVolume;

	/**
	* @access private
	* @var int
	*/
	private $lScale;

	/**
	* @access private
	* @var string
	*/
	private $lVLCExecutable;

	/**
	* @access private
	* @var int
	*/
	private $lChannels;
	
	
	/**
	* @access private
	* @var array
	*/
	private $lProfiles;
	
	private $lSegmentLength;
	
	private $lDVRLength;
	
	private $liPhoneDir;

	/**
	* Constructor. This creates a VLC Server object. Some defaults are already set. These are: volume, scale and audio channels. It alse detects if it is running on Windows or Linux.
	* @return VLCServer
	*/
	function __construct() {		
		$this->lProfiles = array();
		
		/*
			Here you can change some codec settings.
			Possible Audio codecs:
				1. mp4a
				2. mp3
				3. mpga
		*/
		
		// HD Profile
		//$this->lProfiles['HD'] = array("name" => "HD", 		"width" => 1280, "height" => 720, "videocodec" => "h264", "videobitrate" => 1200, "videofps" => 25, "audiocodec" => "mp4a", "audiobitrate" => 192, "audiosamplerate" => 48000);
		// Main Profile
		$this->lProfiles['SD'] = array("name" => "SD", 		"width" => 640, "height" => 360, "videocodec" => "h264", "videobitrate" => 512, "videofps" => 25, "audiocodec" => "mp4a", "audiobitrate" => 128, "audiosamplerate" => 48000);
		// Mobile Profile
		$this->lProfiles['Mobile'] = array("name" => "Mobile", 	"width" => 320, "height" => 180, "videocodec" => "h264", "videobitrate" => 200, "videofps" => 25, "audiocodec" => "mp4a", "audiobitrate" => 96, "audiosamplerate" => 48000);
		
		$this->lVolume = 80;
		$this->lScale = 0.75;
		$this->lChannels = 2;
		$this->lSegmentLength = 10;
		$this->lDVRLength = Settings::getDVRLength();
		
		$this->liPhoneDir = getcwd();
		
		$this->lVLCExecutable = Settings::getVLCExecutable();	
	}

	/**
	* Start the VLC Server by executing the vlc command line. You have to set all options before starting this server. If the server is started, you should be able to connect to it and watch some tv.
	* @param boolean $pDebug
	* @return string
	*/
	public function startServer($pStreamURL,$pChannelObj) {
		global $gDebugObj;
		$pStreamURL = str_replace('name=zap','name=' . $pChannelObj->getName() ,$pStreamURL);
		$lCurrentStatus = $this->getCurrentStatus();
		if ($lCurrentStatus["streamurl"] != $pStreamURL) {
			$this->stopServer();			
			$lCMD = $this->lVLCExecutable . " \"" . $pStreamURL . "\" vlc://quit ";
			if (Settings::getVLCAudioLanguage() != '') {
				$lAudioTracks = explode(',',Settings::getVLCAudioLanguage());
				if (is_numeric($lAudioTracks[0])) {
					$lCMD .= ' --audio-track=' . $lAudioTracks[0];
				} else {
					$lCMD .= ' --audio-language=' . Settings::getVLCAudioLanguage();
				}
			}
			if (Settings::getVLCSubtitleLanguage() != '') {
				$lSubTracks = explode(',',Settings::getVLCSubtitleLanguage());
				if (is_numeric($lSubTracks[0])) {
					$lCMD .= ' --sub-track=' . $lSubTracks[0];
				} else {
					$lCMD .= ' --sub-language=' . Settings::getVLCSubtitleLanguage();
				}
			}
			if (is_object($pChannelObj) && get_class($pChannelObj) == 'Movie' && (($lSubTitleFile = $pChannelObj->getSubtitle(Settings::getVLCSubtitleLanguage())) != '')) {
				$lCMD .= ' --sub-file=' . $pChannelObj->getSubtitle(Settings::getVLCSubtitleLanguage());
			}
			
			$lPortNumbers = 20000;
			$lTransCode = "#duplicate{";
			
			if ($pChannelObj->isHD()) {
				// Only allow one bitrate when the source is HD
				$this->lProfiles = array('SD' => $this->lProfiles['SD']);
			}
			
			foreach ($this->lProfiles as $lProfileName => $lProfile) {
				$lCodecParams = "";
				switch ($lProfile["videocodec"]) {
					case "h264":
					/*
						$lCodecParams = "venc=x264{aud,keyint=10,nocabac,profile=baseline,level=31,idrint=10,bframes=0,ref=1,qpmax=51,qpmin=10,me=hex,subq=6,subme=7,qcomp=0.6,fps=" . ($lProfile["videofps"]) . "},";
						
						$lCodecParams = "venc=x264{vbv-bufsize=500,partitions=all,level=41,no-cabac,subme=7,threads=4,ref=2,mixed-refs=1,bframes=0,min-keyint=1,keyint=50,trellis=2,direct=auto,qcomp=0.0,qpmax=51},";
						
						*/
						
						//http://veetle.com/index.php/article/view/x264
						$lCodecParams = "venc=x264{aud,keyint=10,nocabac,profile=baseline,level=31,idrint=10,bframes=16,b-adapt=1,ref=3,qpmax=51,qpmin=10,me=hex,merange=16,subq=6,subme=7,qcomp=0.6,fps=" . ($lProfile["videofps"]) . "},";
					
						// http://www.wowza.com/forums/showthread.php?7916-Iphone-Wowza-VLC-Camera
						//$lCodecParams = "venc=x264{aud,keyint=60,idrint=30,profile=baseline,level=30,nocabac,qpmax=35,qpmin=10,me=hex,merange=16,subme=1,qcomp=0.6,vbv-bufsize=10,fps=20},";
					
						//$lCodecParams = "venc=x264{aud,keyint=" . ($lProfile["videofps"]*2) . ",min-keyint=" . ($lProfile["videofps"]*2) . ",profile=baseline,level=31,bframes=0,ref=5,qpmax=51,qpmin=10,me=hex,subq=7,subme=7,qcomp=0.6,fps=" . ($lProfile["videofps"]) . "},";
					
						// http://wiki.videolan.org/Documentation:Streaming_HowTo/Streaming_for_the_iPhone
						//$lCodecParams = "venc=x264{aud,profile=baseline,level=30,keyint=30,bframes=0,ref=1,nocabac},";
					
						//$lCodecParams = '';
					break;
				}
				$lTransCode .= "dst={transcode{threads=2,deinterlace,vcodec=". $lProfile["videocodec"] . "," . $lCodecParams . "vb=". $lProfile["videobitrate"] . ",scale=". $this->lScale .",acodec=". $lProfile["audiocodec"] .",ab=" . $lProfile["audiobitrate"] . ",channels=". $this->lChannels .",samplerate=" . $lProfile["audiosamplerate"] . ",width=". $lProfile["width"] .",height=". $lProfile["height"] . ",audio-sync,fps=" . ($lProfile["videofps"]) . ",soverlay}";
				$lTransCode .= ":duplicate{";
			
				$lTransCode .= "dst=gather:std{access=livehttp{seglen=" . $this->lSegmentLength . ",delsegs=true,numsegs=" . ($this->lDVRLength / $this->lSegmentLength). ",index=" . $this->liPhoneDir . "/iphone/stream/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfile["name"] . ".m3u8,index-url=" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfile["name"] . "-########.ts,ratecontrol=true},mux=" . ($lProfile["audiocodec"] == 'mp4a' ? 'ts' : 'raw') . "{use-key-frames},dst=" . $this->liPhoneDir . "/iphone/stream/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfileName . "-########.ts},";
				
				if (count($this->lProfiles) == 1 || $lProfileName == 'Mobile') {
					// RTSP Mobile server only when there is one source
					// OR when the latest bit rate is reached
					$lTransCode .= "dst=rtp{dst=". Settings::getVLCPlayerIP(true) .",ttl=20,sdp=rtsp://:". Settings::getVLCPort() . "/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_Mobile.sdp},";
				}
				
				$lTransCode = substr(trim($lTransCode),0,-1) . "}},";				
				$lPortNumbers += 10;
			}
			
			$lTransCode = substr(trim($lTransCode),0,-1) . "}";
			
			$lTransCode = "\"" . $lTransCode . "\"";
			$lCMD .= " --sout=" . $lTransCode;
			sleep(1); // Wait a bit to make sure that the old VLC process is killed before starting a new one
			Utils::execute($lCMD);
			if (Settings::getDebug()) $gDebugObj->setDebugMessage('startServer',"VLC Command:\n".$lCMD);
		}
		
		if (!Utils::isMobileDevice() || Utils::isiPhone()) $this->setIphoneStream($pChannelObj);
		return $this->getPlayerUrl($pChannelObj);
	}
	
	public function getPlayerUrl($pChannelObj) {
		if (Utils::isMobileDevice() && !Utils::isiPhone()) {
			$lReturnValue = "rtsp://" . Settings::getVLCPlayerIP(true) . ":" . Settings::getVLCPort() . "/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_Mobile.sdp";
		} else {
			$lReturnValue = 'iphone/stream/' . $this->vlcStreamUrlFormat($pChannelObj->getName(true)) . '.m3u8';
		}
		return $lReturnValue;
	}

	public function getCurrentStatus() {
		$lReturnArray = array();
		$lReturnArray["channel"] = "";
		$lReturnArray["streamurl"] = "";
		$lReturnArray["procesid"] = -1;
		
		$lData = trim(shell_exec("ps ax | grep vlc://quit | grep -v grep"));
		if ($lData != '') {
			$lReturnArray["procesid"] = substr($lData,0,stripos($lData,' '));
			$lStartPos = stripos($lData,'dummy') + 6;
			$lEndPos = stripos($lData,'vlc://quit');
			$lReturnArray["streamurl"] = trim(substr($lData,$lStartPos,$lEndPos-$lStartPos));
			if (substr($lReturnArray["streamurl"],0,7) == "http://") {
				$lChannelData = Utils::convertUrlQuery(parse_url($lReturnArray["streamurl"],PHP_URL_QUERY));
				if (!empty($lChannelData['ref'])) {
					$lReturnArray["channel"] = $lChannelData['ref'];
				} else if (!empty($lChannelData['file'])) {
					$lReturnArray["channel"] = $lChannelData['file'];
				}
			} else {
				$lReturnArray["channel"] = $lReturnArray["streamurl"];
			}
			$lReturnArray["channel"] = str_replace("%20"," ",$lReturnArray["channel"]);
		}
		return $lReturnArray;
	}

	/**
	* Stop the VLC Server. This is handy to spare some system resources.
	*/
	public function stopServer() {
		$lCurrentStatus = $this->getCurrentStatus();
		$lCMD = '';
		if ($lCurrentStatus['procesid'] > 0) $lCMD = "kill -9 " . $lCurrentStatus['procesid'] . ";";
		$lCMD .= "rm " . $this->liPhoneDir . "/iphone/stream/*";
		Utils::execute($lCMD);
	}

	public function setIphoneStream($pChannelObj) {
		global $gDebugObj;
		if (strpos($pChannelObj->getID(),':19:') !== false && count($this->lProfiles) > 1) {
			// Only allow one bitrate when the source is HD
			$this->lProfiles = array('SD' => $this->lProfiles['SD']);
		}
		if (Settings::getDebug()) $gDebugObj->setDebugMessage('setIphoneStream',"Start checking for iOS playlists... Amount of profiles: " . count($this->lProfiles));
		for ($i = 0; $i < 30; $i++) {
			$lMultiBitrate = "#EXTM3U\n";
			$lProfilesCounter = 0;
			if (Settings::getDebug()) $gDebugObj->setDebugMessage('setIphoneStream',"Checking profile: " . $lProfilesCounter);
			foreach ($this->lProfiles as $lProfileName => $lProfile) {
				if (file_exists($this->liPhoneDir . "/iphone/stream/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfile["name"] . ".m3u8")) {
					$lMultiBitrate .= "#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=" . round(($lProfile["videobitrate"]+$lProfile["audiobitrate"]) * 1024) . ",RESOLUTION=" . $lProfile["width"] . "x" . $lProfile["height"] . "\n" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_" . $lProfileName . ".m3u8\n";
					$lProfilesCounter++;
				}
			}
			if (Settings::getDebug()) $gDebugObj->setDebugMessage('setIphoneStream',"After $i seconds, amount of profiles done: " . $lProfilesCounter);
			if ($lProfilesCounter == count($this->lProfiles)) {
				file_put_contents($this->liPhoneDir . "/iphone/stream/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . ".m3u8",$lMultiBitrate);
				sleep(1);
				break;
			}
			else sleep(1);
		} 
	}

	private function vlcStreamUrlFormat($pUrl) {
		$pUrl = trim($pUrl);
		return str_replace(array(" ","'"),array("_",""),preg_replace("/[^a-zA-Z0-9_\s]/","",$pUrl));
	}

	/**
	* Check if the server is still running.
	* @return boolean
	*/
	public function isServerRunning() {
		$lCurrentStatus = $this->getCurrentStatus();
		return $lCurrentStatus['procesid'] > 0;
	}
}
